@@ -108,7 +108,8 @@
 (defun shade-plot ()
   (plsdev gdev)
   (plinit)
-  (plenv -1 1 -1 1 0 0)
+;  (plenv -1 1 -1 1 0 0)
+  (plenv 0.0 35.0 0.0 45.0 0 0)
   (plshades (example-matrix 35 45 #'example-func-1) -1 1 -1 1 (make-levels 20 -1.0 1.0) 2 1 1 nil)
   (plcol0 1)
   (plbox "bcnst" 0 0 "bcnstv" 0 0)
