@@ -28,6 +28,19 @@
 (pl-defcfun ("c_pladv" pladv) :void 
 	    (page plint))
 
+(pl-defcfun ("c_plarc" plarc) :void
+	    (x plflt)
+	    (y plflt)
+	    (a plflt)
+	    (b plflt)
+	    (angle1 plflt)
+	    (angle2 plflt)
+	    (fill plbool))
+	    
+;;
+;; This function is deprecated and may dissappear at some point
+;;
+
 (pl-defcfun ("plarrows" plarrows) :void
 	    (u *plflt n)
 	    (v *plflt n)
@@ -39,8 +52,8 @@
 	    (dy plflt))
 
 (defun plvect (u v scale &optional (gridx nil) (gridy nil))
-  (with-plcgrid (gridx gridy (array-dimension u 0) (array-dimension u 1))  ; this macro creates the variable plcgrid
-    (pl-plvect u v scale plcgrid)))
+  (with-plcgrid (plc-grid gridx gridy (array-dimension u 0) (array-dimension u 1))  ; this macro creates the variable plcgrid
+    (pl-plvect u v scale plc-grid)))
 
 (export 'plvect (package-name *package*))
 
@@ -70,10 +83,19 @@
 	(nysub plint))
 
 (pl-defcfun ("c_plbin" plbin) :void 
-	(nbin plint)
-	(x *plflt nbin)
-	(y *plflt nbin)
-	(opt plint))
+	    (nbin plint)
+	    (x *plflt nbin)
+	    (y *plflt nbin)
+	    (opt plint))
+
+(pl-defcfun ("c_plbtime" plbtime) :void
+	    (year *plint 1)
+	    (month *plint 1)
+	    (day *plint 1)
+	    (hour *plint 1)
+	    (min *plint 1)
+	    (sec *plflt 1)
+	    (ctime plflt))
 
 (pl-defcfun ("c_plbop" plbop) :void)
 
@@ -114,6 +136,19 @@
 (pl-defcfun ("c_plcol1" plcol1) :void 
 	    (col1 plflt))
 
+(pl-defcfun ("c_plconfigtime" plconfigtime) :void
+	    (scale plflt)
+	    (offset1 plflt)
+	    (offset2 plflt)
+	    (ccontrol plint)
+	    (ifbtime-offset plbool)
+	    (year plint)
+	    (month plint)
+	    (day plint)
+	    (hour plint)
+	    (min plint)
+	    (sec plflt))
+
 
 ;;
 ;; Coming on this a couple months later I had a pretty hard time figuring
@@ -150,8 +185,8 @@
 ;;
 
 (defun plcont (f kx lx ky ly clevel &optional (gridx nil) (gridy nil))
-  (with-plcgrid (gridx gridy (array-dimension f 0) (array-dimension f 1))
-    (pl-plcont f kx lx ky ly clevel plcgrid)))
+  (with-plcgrid (plc-grid gridx gridy (array-dimension f 0) (array-dimension f 1))
+    (pl-plcont f kx lx ky ly clevel plc-grid)))
 
 (export 'plcont (package-name *package*))
 
@@ -188,8 +223,8 @@
 
 (defun plfcont (f2eval-data nx ny kx lx ky ly clevel &optional (gridx nil) (gridy nil))
   (let ((c-f2eval-data (init-f2eval-data f2eval-data nx ny)))
-    (with-plcgrid (gridx gridy nx ny)
-      (pl-plfcont c-f2eval-data nx ny kx lx ky ly clevel plcgrid))
+    (with-plcgrid (plc-grid gridx gridy nx ny)
+      (pl-plfcont c-f2eval-data nx ny kx lx ky ly clevel plc-grid))
     (free-f2eval-data f2eval-data nx ny)))
 
 (export 'plfcont (package-name *package*))
@@ -212,6 +247,15 @@
 	    (iplsr plint)
 	    (flags plint))
 
+(pl-defcfun ("c_plctime" plctime) :void
+	    (year plint)
+	    (month plint)
+	    (day plint)
+	    (hour plint)
+	    (min plint)
+	    (sec plflt)
+	    (ctime *plflt 1))
+
 (pl-defcfun ("pldid2pc" pldid2pc) :void 
 	(xmin *plflt :in-out)
 	(ymin *plflt :in-out)
@@ -292,11 +336,24 @@
 	    (g *plint 1)
 	    (b *plint 1))
 
+(pl-defcfun ("c_plgcol0a" plgcol0a) :void 
+	    (icol0 plint)
+	    (r *plint 1)
+	    (g *plint 1)
+	    (b *plint 1)
+	    (a *plflt 1))
+
 (pl-defcfun ("c_plgcolbg" plgcolbg) :void 
 	    (r *plint 1)
 	    (g *plint 1)
 	    (b *plint 1))
 
+(pl-defcfun ("c_plgcolbga" plgcolbga) :void 
+	    (r *plint 1)
+	    (g *plint 1)
+	    (b *plint 1)
+	    (a *plflt 1))
+
 (pl-defcfun ("c_plgcompression" plgcompression) :void 
 	    (compression *plint 1))
 
@@ -318,17 +375,22 @@
 	    (p_xmax *plflt 1)
 	    (p_ymax *plflt 1))
 
-(pl-defcfun ("c_plgfci" plgfci) :void 
-	    (pfci *plunicode 1))
-
 (pl-defcfun ("c_plgfam" plgfam) :void 
 	    (p_fam *plint 1)
 	    (p_num *plint 1)
 	    (p_bmax *plint 1))
 
+(pl-defcfun ("c_plgfci" plgfci) :void 
+	    (pfci *plunicode 1))
+
 (pl-defcfun ("c_plgfnam" plgfnam) :void 
 	    (fnam plstr 80))
 
+(pl-defcfun ("c_plgfont" plgfont) :void 
+	    (p_family *plint 1)
+	    (p_style *plint 1)
+	    (p_weight *plint 1))
+
 (pl-defcfun ("c_plglevel" plglevel) :void 
 	    (p_level *plint 1))
 
@@ -342,6 +404,12 @@
 
 (pl-defcfun ("c_plgra" plgra) :void)
 
+(pl-defcfun ("c_plgradient" plgradient) :void 
+	    (n plint)
+	    (x *plflt n)
+	    (y *plflt n)
+	    (angle plflt))
+
 (defun plgriddata (x y z xg yg type data)
   (let ((sx (length x))
 	(sy (length y))
@@ -416,6 +484,7 @@
 	    (nbin plint)
 	    (opt plint))
 
+;; deprecated
 (pl-defcfun ("c_plhls" plhls) :void 
 	    (h plflt)
 	    (l plflt)
@@ -500,7 +569,7 @@
 	    (nlevel plint))
 
 (pl-defcfun ("c_plmkstrm" plmkstrm) :void 
-	    (p_strm plint :out))
+	    (p_strm *plint 1))
 
 (pl-defcfun ("c_plmtex" plmtex) :void 
 	    (side plstr)
@@ -551,8 +620,15 @@
 
 (pl-defcfun ("c_plpat" plpat) :void 
 	    (nlin plint)
-	    (inc *plint nlin)
-	    (del *plint nlin))
+	    (inc *plint n)
+	    (del *plint n))
+
+(pl-defcfun ("c_plpath" plpath) :void
+	    (n plint)
+	    (x1 plflt)
+	    (y1 plflt)
+	    (x2 plflt)
+	    (y2 plflt))
 
 (pl-defcfun ("c_plpoin" plpoin) :void 
 	    (n plint)
@@ -603,13 +679,18 @@
 	    (just plflt)
 	    (text plstr))
 
+
+(pl-defcfun ("c_plrandd" plrandd) :double)
+
 (pl-defcfun ("c_plreplot" plreplot) :void)
 
+;; deprecated
 (pl-defcfun ("c_plrgb" plrgb) :void 
 	    (r plflt)
 	    (g plflt)
 	    (b plflt))
 
+;; deprecated
 (pl-defcfun ("c_plrgb1" plrgb1) :void 
 	    (r plint)
 	    (g plint)
@@ -633,6 +714,13 @@
 	    (b *plint ncol0)
 	    (ncol0 plint))
 
+(pl-defcfun ("c_plscmap0a" plscmap0a) :void 
+	    (r *plint ncol0)
+	    (g *plint ncol0)
+	    (b *plint ncol0)
+	    (a *plflt ncol0)
+	    (ncol0 plint))
+
 (pl-defcfun ("c_plscmap0n" plscmap0n) :void 
 	    (ncol0 plint))
 
@@ -642,6 +730,13 @@
 	    (b *plint ncol1)
 	    (ncol1 plint))
 
+(pl-defcfun ("c_plscmap1a" plscmap1a) :void 
+	    (r *plint ncol1)
+	    (g *plint ncol1)
+	    (b *plint ncol1)
+	    (a *plflt ncol1)
+	    (ncol1 plint))
+
 (pl-defcfun ("c_plscmap1l" plscmap1l) :void 
 	    (itype plbool)
 	    (npts plint)
@@ -649,6 +744,16 @@
 	    (coord1 *plflt npts)
 	    (coord2 *plflt npts)
 	    (coord3 *plflt npts)
+	    (rev *plbool n))
+
+(pl-defcfun ("c_plscmap1la" plscmap1la) :void 
+	    (itype plbool)
+	    (npts plint)
+	    (intensity *plflt npts)
+	    (coord1 *plflt npts)
+	    (coord2 *plflt npts)
+	    (coord3 *plflt npts)
+	    (coord4 *plflt npts)
 	    (rev *plbool npts))
 
 (pl-defcfun ("c_plscmap1n" plscmap1n) :void 
@@ -660,11 +765,24 @@
 	    (g plint)
 	    (b plint))
 
+(pl-defcfun ("c_plscol0a" plscol0a) :void 
+	    (icol0 plint)
+	    (r plint)
+	    (g plint)
+	    (b plint)
+	    (a plflt))
+
 (pl-defcfun ("c_plscolbg" plscolbg) :void 
 	    (r plint)
 	    (g plint)
 	    (b plint))
 
+(pl-defcfun ("c_plscolbga" plscolbga) :void 
+	    (r plint)
+	    (g plint)
+	    (b plint)
+	    (a plflt))
+
 (pl-defcfun ("c_plscolor" plscolor) :void 
 	    (color plint))
 
@@ -703,6 +821,9 @@
 	    (xmax plflt)
 	    (ymax plflt))
 
+(pl-defcfun ("c_plseed" plseed) :void 
+	    (seed plint))
+
 (pl-defcfun ("c_plsesc" plsesc) :void 
 	    (esc plchar))
 
@@ -718,7 +839,6 @@
 ;; is preserved until the next time this function is called
 
 (let ((c-fname nil))
-  
   (defun plsfnam (fname)
     (when c-fname
       (foreign-string-free c-fname))
@@ -730,18 +850,30 @@
 (defcfun ("c_plsfnam" c-plsfnam) :void 
   (fnam plstr))
 
+(pl-defcfun ("c_plsfont" plsfont) :void 
+	    (family plint)
+	    (style plint)
+	    (weight plint))
+
+;;
 ;; Note that for these functions you have no choice when it comes to the fill function,
 ;; as this is what the plplot docs suggest. You can set a user defined exclude function 
 ;; and a user defined contour function by setting my-defined-fn or my-pltr-fn inside
 ;; their respective closures.
+;;
+;; Note also that left, right, bottom and top are ignored as it is (as currently arranged)
+;; impossible to set the pltr-fn to null due to the fact that it is a callback.
+;;
 
 ;; use the no exclusion function or set your own with this closure
 
 (callback-closure defined-fn #'(lambda(x y) (declare (ignore x y)) 1) plint (x plflt) (y plflt))
 
 (defun plshade (a left right bottom top shade-min shade-max sh-cmap sh-color sh-width min-color min-width max-color max-width rectangular &optional (gridx nil) (gridy nil))
-  (with-plcgrid (gridx gridy (array-dimension a 0) (array-dimension a 1))
-    (pl-plshade a left right bottom top shade-min shade-max sh-cmap sh-color sh-width min-color min-width max-color max-width rectangular plcgrid)))
+  (if (and gridx gridy)
+      (with-plcgrid (plc-grid gridx gridy (array-dimension a 0) (array-dimension a 1))
+	(pl-plshade a left right bottom top shade-min shade-max sh-cmap sh-color sh-width min-color min-width max-color max-width rectangular plc-grid))
+      (pl-plshade-simple a left right bottom top shade-min shade-max sh-cmap sh-color sh-width min-color min-width max-color max-width rectangular (null-pointer) (null-pointer))))
 
 (export 'plshade (package-name *package*))
 
@@ -768,12 +900,37 @@
 	    (pltr-fn plfunc)
 	    (pltr-data plpointer))
 
+(pl-defcfun ("c_plshade" pl-plshade-simple) :void
+	    (a **plflt (nx ny))
+	    (nx plint)
+	    (ny plint)
+	    (defined-fn plfunc)
+	    (left plflt)
+	    (right plflt)
+	    (bottom plflt)
+	    (top plflt)
+	    (shade-min plflt)
+	    (shade-max plflt)
+	    (sh-cmap plint)
+	    (sh-color plflt)
+	    (sh-width plint)
+	    (min-color plint)
+	    (min-width plint)
+	    (max-color plint)
+	    (max-width plint)
+	    (plfill-fn plfunc)
+	    (rectangular plbool)
+	    (pltr-fn plpointer)
+	    (pltr-data plpointer))
+
 (defun plshades (a xmin xmax ymin ymax clevel fill-width cont-color cont-width rectangular &optional (gridx nil) (gridy nil))
-  (with-plcgrid (gridx gridy (array-dimension a 0) (array-dimension a 1))
-    (pl-plshades a xmin xmax ymin ymax clevel fill-width cont-color cont-width rectangular plcgrid)))
+  (if (and gridx gridy)
+      (with-plcgrid (plc-grid gridx gridy (array-dimension a 0) (array-dimension a 1))
+	(pl-plshades a xmin xmax ymin ymax clevel fill-width cont-color cont-width rectangular plc-grid))
+      (pl-plshades-simple a xmin xmax ymin ymax clevel fill-width cont-color cont-width rectangular (null-pointer) (null-pointer))))
 
 (export 'plshades (package-name *package*))
-    
+
 (pl-defcfun ("c_plshades" pl-plshades) :void
 	    (a **plflt (nx ny))
 	    (nx plint)
@@ -793,6 +950,24 @@
 	    (pltr-fn plfunc)
 	    (pltr-data plpointer))
 
+(pl-defcfun ("c_plshades" pl-plshades-simple) :void
+	    (a **plflt (nx ny))
+	    (nx plint)
+	    (ny plint)
+	    (defined-fn plfunc)
+	    (xmin plflt)
+	    (xmax plflt)
+	    (ymin plflt)
+	    (ymax plflt)
+	    (clevel *plflt nlevel)
+	    (nlevel plint)
+	    (fill-width plint)
+	    (cont-color plint)
+	    (cont-width plint)
+	    (plfill-fn plfunc)
+	    (rectangular plbool)
+	    (pltr-fn plpointer)
+	    (pltr-data plpointer))
 
 ;; The "friendly" form of this one was deemed to be adequately covered by the above 
 ;; functions, so only the "cffi" version of the function is provided.
@@ -834,11 +1009,22 @@
 ; PLINT max_color, PLINT max_width, 	 void (*fill) (PLINT, PLFLT *, PLFLT *), PLBOOL rectangular, 	 
 ; void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer), 	 PLPointer pltr_data
 
-(pl-defcfun ("c_plsmaj" plsmaj) :void 
+;(pl-defcfun ("c_plsfam" plsfam) :void 
+
+(pl-defcfun ("c_plslabelfunc" plslabelfunc) :void 
+	    (label-fn plfunc)
+	    (label-data plpointer))
+
+(callback-closure label-fn #'(lambda(axis value label length data)
+			       (declare (ignore axis value data))
+			       (lisp-string-to-foreign "not set" label length))					
+		  :void 
+		  (axis plint) (value plflt) (label plstr) (length plint) (data plpointer))
+
+(pl-defcfun ("c_plsmaj" plsmaj) :void
 	    (def plflt)
 	    (scale plflt))
 
-
 ;;
 ;; plsmem is broken into three parts:
 ;;
@@ -901,6 +1087,13 @@
 	    (xoff plint)
 	    (yoff plint))
 
+(pl-defcfun ("c_plspal0" plspal0) :void
+	    (filename plstr))
+
+(pl-defcfun ("c_plspal1" plspal1) :void
+	    (filename plstr)
+	    (interpolate plbool))
+
 (pl-defcfun ("c_plspause" plspause) :void 
 	    (pause plbool))
 
@@ -924,6 +1117,24 @@
 	    (nx plint)
 	    (ny plint))
 
+(pl-defcfun ("c_plstransform" plstransform) :void 
+	    (transform-fn plfunc)
+	    (data plpointer))
+
+(callback-closure transform-fn #'(lambda (x y tx ty pltr-data)
+				   (declare (ignore pltr-data))
+				   (setf (mem-aref tx :double) (coerce x 'double-float)
+					 (mem-aref ty :double) (coerce y 'double-float)))
+		  :void 
+		  (x plflt) (y plflt) (tx *plflt) (ty *plflt) (pltr-data plpointer))
+
+;; This is a lame hack to make it easier for the user to reset
+;; PLplots universal coordinate transform function.
+(defun pl-reset-transform ()
+  (c-plstransform (null-pointer) (null-pointer)))
+
+(export 'pl-reset-transform (package-name *package*))
+
 (pl-defcfun ("c_plstripa" plstripa) :void 
 	    (id plint)
 	    (pen plint)
@@ -978,7 +1189,28 @@
 (pl-defcfun ("c_plstripd" plstripd) :void 
 	    (id plint))
 
-(pl-defcfun ("plimage" plimage) :void 
+(pl-defcfun ("c_plimagefr" pl-plimagefr) :void 
+	    (idata **plflt (nx ny))
+	    (nx plint)
+	    (ny plint)
+	    (xmin plflt)
+	    (xmax plflt)
+	    (ymin plflt)
+	    (ymax plflt)
+	    (zmin plflt)
+	    (zmax plflt)
+	    (valuemin plflt)
+	    (valuemax plflt)
+	    (pltr-fn plfunc)
+	    (pltr-data plpointer))
+
+(defun plimagefr (idata xmin xmax ymin ymax zmin zmax valuemin valuemax &optional (gridx nil) (gridy nil))
+  (with-plcgrid (plc-grid gridx gridy (array-dimension idata 0) (array-dimension idata 1))
+    (pl-plimagefr idata xmin xmax ymin ymax zmin zmax valuemin valuemax plc-grid)))
+
+(export 'plimagefr (package-name *package*))
+
+(pl-defcfun ("c_plimage" plimage) :void 
 	    (data **plflt (nx ny))
 	    (nx plint)
 	    (ny plint)
@@ -995,8 +1227,8 @@
 
 (pl-defcfun ("c_plstyl" plstyl) :void 
 	    (nms plint)
-	    (mark *plint nms)
-	    (space *plint nms))
+	    (mark *plint n)
+	    (space *plint n))
 
 (pl-defcfun ("c_plsurf3d" plsurf3d) :void 
 	    (x *plflt nx)
@@ -1008,6 +1240,30 @@
 	    (clevel *plflt nlevel)
 	    (nlevel plint))
 
+;; See x08l.lisp for an example of how to use this function.
+(pl-defcfun ("plfsurf3d" plfsurf3d) :void
+	    (x *plflt nx)
+	    (y *plflt ny)
+	    (zops plpointer)
+	    (zp plpointer)
+	    (nx plint)
+	    (ny plint)
+	    (opt plint)
+	    (clevel *plflt nlevel)
+	    (nlevel plint))
+
+;; These are helper functions for plf... functions, they
+;; return a pointer to a "zops" function.
+(defcfun "plf2ops_c" :pointer)
+(defcfun "plf2ops_grid_c" :pointer)
+(defcfun "plf2ops_grid_row_major" :pointer)
+(defcfun "plf2ops_grid_col_major" :pointer)
+
+(export 'plf2ops-c (package-name *package*))
+(export 'plf2ops-grid-c (package-name *package*))
+(export 'plf2ops-grid-row-major (package-name *package*))
+(export 'plf2ops-grid-col-major (package-name *package*))
+
 (pl-defcfun ("c_plsurf3dl" plsurf3dl) :void 
 	    (x *plflt nx)
 	    (y *plflt ny)
@@ -1051,6 +1307,9 @@
 
 (pl-defcfun ("c_pltext" pltext) :void)
 
+(pl-defcfun ("c_pltimefmt" pltimefmt) :void
+	    (fmt plstr))
+			   
 (pl-defcfun ("c_plvasp" plvasp) :void 
 	    (aspect plflt))
 
@@ -1378,6 +1637,10 @@
 (pl-defcfun ("plgesc" plgesc) :void 
 	    (p_esc *plchar 1))
 
+(pl-defcfun ("pl_cmd" pl-cmd) :void
+	    (op plint)
+	    (ptr plpointer))
+
 ;(defcfun ("pl_cmd" pl-cmd) :void 
 ;	PLINT op, void *ptr
 ;
@@ -1414,12 +1677,8 @@
 	(nx plint)
 	(ny plint))
 
-
-;; Not sure about the utility of this one. f must be an appropriately 
-;; created foreign pointer, such as one created by make-matrix
-
 (pl-defcfun ("plMinMax2dGrid" plminmax2dgrid) :void 
-	    (f plpointer)
+	    (f **plflt (nx ny))
 	    (nx plint)
 	    (ny plint)
 	    (fmax *plflt 1)
